@@ -422,6 +422,7 @@ escl_devcaps_parse (devcaps *caps, const char *xml_text, size_t xml_len)
 {
     error  err = NULL;
     xml_rd *xml;
+    bool   quirk_canon_iR2625_2630 = false;
 
     /* Parse capabilities XML */
     err = xml_rd_begin(&xml, xml_text, xml_len, NULL);
@@ -436,7 +437,13 @@ escl_devcaps_parse (devcaps *caps, const char *xml_text, size_t xml_len)
 
     xml_rd_enter(xml);
     for (; !xml_rd_end(xml); xml_rd_next(xml)) {
-        if (xml_rd_node_name_match(xml, "scan:Platen")) {
+        if (xml_rd_node_name_match(xml, "pwg:MakeAndModel")) {
+            const char *m = xml_rd_node_value(xml);
+
+            if (!strcmp(m, "Canon iR2625/2630")) {
+                quirk_canon_iR2625_2630 = true;
+            }
+        } else if (xml_rd_node_name_match(xml, "scan:Platen")) {
             xml_rd_enter(xml);
             if (xml_rd_node_name_match(xml, "scan:PlatenInputCaps")) {
                 err = escl_devcaps_source_parse(xml,
@@ -483,6 +490,26 @@ escl_devcaps_parse (devcaps *caps, const char *xml_text, size_t xml_len)
         goto DONE;
     }
 
+    /* Apply quirks, if any */
+    if (quirk_canon_iR2625_2630) {
+        /* This device announces resolutions up to 600 DPI,
+         * but actually doesn't support more that 300
+         *
+         * https://oip.manual.canon/USRMA-4209-zz-CSL-2600-enUV/contents/devu-apdx-sys_spec-send.html?search=600
+         *
+         * See #57 for details
+         */
+        for (id_src = (ID_SOURCE) 0; id_src < NUM_ID_SOURCE; id_src ++) {
+            devcaps_source *src = caps->src[id_src];
+            if (src != NULL &&
+                /* paranoia: array won't be empty after quirk applied */
+                sane_word_array_len(src->resolutions) > 0 &&
+                src->resolutions[1] <= 300) {
+                sane_word_array_bound(src->resolutions, 0, 300);
+            }
+        }
+    }
+
 DONE:
     if (err != NULL) {
         devcaps_reset(caps);
@@ -514,6 +541,17 @@ escl_devcaps_decode (const proto_ctx *ctx, devcaps *caps)
     return escl_devcaps_parse(caps, data->bytes, data->size);
 }
 
+/* Fix Location: URL
+ *
+ * Can be used as http_query_onredir() callback
+ */
+static void
+escl_scan_fix_location (void *p, http_uri *uri, const http_uri *orig_uri)
+{
+    (void) p;
+    http_uri_fix_host(uri, orig_uri, "localhost");
+}
+
 /* Initiate scanning
  */
 static http_query*
@@ -595,6 +633,7 @@ escl_scan_query (const proto_ctx *ctx)
      * many devices.
      */
     http_query_set_request_header(query, "Host", "localhost");
+    http_query_onredir(query, escl_scan_fix_location);
 
     return query;
 }
@@ -626,13 +665,14 @@ escl_scan_decode (const proto_ctx *ctx)
     }
 
     /* Validate and save location */
-    uri = http_uri_new_relative(ctx->base_uri, location, true, true);
+    uri = http_uri_new_relative(ctx->base_uri, location, true, false);
     if (uri == NULL) {
         err = eloop_eprintf("ScanJobs request: invalid location received");
         goto ERROR;
     }
 
-    result.data.location = str_dup(http_uri_get_path(uri));
+    escl_scan_fix_location(NULL, uri, http_query_uri(ctx->query));
+    result.data.location = str_dup(http_uri_str(uri));
     http_uri_free(uri);
 
     result.next = PROTO_OP_LOAD;
