@@ -1417,6 +1417,12 @@ http_uri_get_path (const http_uri *uri);
 void
 http_uri_set_path (http_uri *uri, const char *path);
 
+/* Fix URI host: if `match` is NULL or uri's host matches `match`,
+ * replace uri's host and port with values taken from the base_uri
+ */
+void
+http_uri_fix_host (http_uri *uri, const http_uri *base_uri, const char *match);
+
 /* Fix IPv6 address zone suffix
  */
 void
@@ -1527,6 +1533,11 @@ http_client_onerror (http_client *client,
 void
 http_client_cancel (http_client *client);
 
+/* Set timeout of all pending queries, if any. Timeout is in milliseconds
+ */
+void
+http_client_timeout (http_client *client, int timeout);
+
 /* Cancel all pending queries with matching address family and uintptr
  */
 void
@@ -1560,6 +1571,14 @@ http_query_new_relative(http_client *client,
         const http_uri *base_uri, const char *path,
         const char *method, char *body, const char *content_type);
 
+/* Set query timeout, in milliseconds. Negative timeout means 'infinite'
+ *
+ * This function may be called multiple times (each subsequent call overrides
+ * a previous one)
+ */
+void
+http_query_timeout (http_query *q, int timeout);
+
 /* For this particular query override on-error callback, previously
  * set by http_client_onerror()
  *
@@ -1570,6 +1589,19 @@ http_query_new_relative(http_client *client,
 void
 http_query_onerror (http_query *q, void (*onerror)(void *ptr, error err));
 
+/* Set on-redirect callback. It is called in a case of HTTP
+ * redirect and may modify the supplied URI
+ */
+void
+http_query_onredir (http_query *q,
+        void (*onredir)(void *ptr, http_uri *uri, const http_uri *orig_uri));
+
+/* Set callback that will be called, when response headers reception
+ * is completed
+ */
+void
+http_query_onrxhdr (http_query *q, void (*onrxhdr)(void *ptr, http_query *q));
+
 /* Submit the query.
  *
  * When query is finished, callback will be called. After return from
@@ -1624,10 +1656,22 @@ const char*
 http_query_status_string (const http_query *q);
 
 /* Get query URI
+ *
+ * It works as http_query_orig_uri() before query is submitted
+ * or after it is completed, and as http_query_real_uri() in
+ * between
+ *
+ * This function is deprecated, use http_query_orig_uri()
+ * or http_query_real_uri() instead
  */
 http_uri*
 http_query_uri (const http_query *q);
 
+/* Get original URI (the same as used when http_query was created)
+ */
+http_uri*
+http_query_orig_uri (const http_query *q);
+
 /* Get real URI, that can differ from the requested URI
  * in a case of HTTP redirection
  */
@@ -1698,6 +1742,12 @@ http_query_foreach_response_header (const http_query *q,
         void (*callback)(const char *name, const char *value, void *ptr),
         void *ptr);
 
+/* Decode response part of the query.
+ * This function is intended for testing purposes, not for regular use
+ */
+error
+http_query_test_decode_response (http_query *q, const void *data, size_t size);
+
 /* HTTP schemes
  */
 typedef enum {
@@ -1823,6 +1873,11 @@ sane_word_array_append (SANE_Word *a, SANE_Word w)
     return a;
 }
 
+/* Drop array elements that outside of specified boundary
+ */
+void
+sane_word_array_bound (SANE_Word *a, SANE_Word min, SANE_Word max);
+
 /* Sort array of SANE_Word in increasing order
  */
 void
